"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
exports.withStylesFunctional = withStylesFunctional;
Object.defineProperty(exports, "withStylesPropTypes", {
  enumerable: true,
  get: function get() {
    return _withStylesPropTypes.withStylesPropTypes;
  }
});
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _react = _interopRequireDefault(require("react"));
var _hoistNonReactStatics = _interopRequireDefault(require("hoist-non-react-statics"));
var _getComponentName = _interopRequireDefault(require("./utils/getComponentName"));
var _useStyles2 = _interopRequireDefault(require("./hooks/useStyles"));
var _detectHooks = _interopRequireDefault(require("./hooks/detectHooks"));
var _emptyStylesFn = _interopRequireDefault(require("./utils/emptyStylesFn"));
var _withStylesPropTypes = require("./withStylesPropTypes");
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { (0, _defineProperty2["default"])(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; } /* eslint-disable no-param-reassign, no-func-assign, react/destructuring-assignment */
/**
 * A higher order function that returns a higher order functional component that injects
 * CSS-in-JS props derived from the react-with-styles theme, interface, and
 * direction provided through the FunctionalWithStylesContext provider.
 *
 * The function should be used as follows:
 * `withStylesFunctional((theme) => styles, options)(Component)`
 *
 * Options can be used to rename the injected props, memoize the component, and flush
 * the styles to the styles tag (or whatever the interface implements as flush) before
 * rendering.
 *
 * Note that this implementation does not use caching for stylesFn(theme) results, so
 * use this if you are not relying on this performance optimization that currently exists
 * in the class implementation of withStyles.
 *
 * @export
 * @param {*} [stylesFn=EMPTY_STYLES_FN]
 * @param {*} [{
 *     stylesPropName = 'styles',
 *     themePropName = 'theme',
 *     cssPropName = 'css',
 *     flushBefore = false,
 *     pureComponent = false,
 *   }={}]
 * @returns a higher order component that wraps the provided component and injects
 * the react-with-styles css, styles, and theme props.
 */
function withStylesFunctional() {
  var stylesFn = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _emptyStylesFn["default"];
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
    _ref$stylesPropName = _ref.stylesPropName,
    stylesPropName = _ref$stylesPropName === void 0 ? 'styles' : _ref$stylesPropName,
    _ref$themePropName = _ref.themePropName,
    themePropName = _ref$themePropName === void 0 ? 'theme' : _ref$themePropName,
    _ref$cssPropName = _ref.cssPropName,
    cssPropName = _ref$cssPropName === void 0 ? 'css' : _ref$cssPropName,
    _ref$flushBefore = _ref.flushBefore,
    flushBefore = _ref$flushBefore === void 0 ? false : _ref$flushBefore;
  stylesFn = stylesFn || _emptyStylesFn["default"];
  if (!(0, _detectHooks["default"])()) {
    throw new ReferenceError('withStylesFunctional() requires React 16.8 or later');
  }

  // The function that wraps the provided component in a wrapper
  // component that injects the withStyles props
  return function withStylesFunctionalHOC(WrappedComponent) {
    var wrappedComponentName = (0, _getComponentName["default"])(WrappedComponent);

    // The wrapper component that injects the withStyles props
    function FunctionalWithStyles(props) {
      var _useStyles = (0, _useStyles2["default"])({
          stylesFn: stylesFn,
          flushBefore: flushBefore
        }),
        css = _useStyles.css,
        styles = _useStyles.styles,
        theme = _useStyles.theme;
      return /*#__PURE__*/_react["default"].createElement(WrappedComponent, (0, _extends2["default"])({}, props, (0, _defineProperty2["default"])((0, _defineProperty2["default"])((0, _defineProperty2["default"])({}, themePropName, theme), stylesPropName, styles), cssPropName, css)));
    }

    // Copy the wrapped component's prop types and default props on FunctionalWithStyles
    if (WrappedComponent.propTypes) {
      FunctionalWithStyles.propTypes = _objectSpread({}, WrappedComponent.propTypes);
      delete FunctionalWithStyles.propTypes[stylesPropName];
      delete FunctionalWithStyles.propTypes[themePropName];
      delete FunctionalWithStyles.propTypes[cssPropName];
    }
    if (WrappedComponent.defaultProps) {
      FunctionalWithStyles.defaultProps = _objectSpread({}, WrappedComponent.defaultProps);
    }
    FunctionalWithStyles.WrappedComponent = WrappedComponent;
    FunctionalWithStyles.displayName = "withStyles(".concat(wrappedComponentName, ")");
    return (0, _hoistNonReactStatics["default"])(FunctionalWithStyles, WrappedComponent);
  };
}
var _default = exports["default"] = withStylesFunctional;